### Have cran package "deSolve" installed


### Source model script. In this case, it is Fitz-Hugh Nagumo Model.R

source("Fitz-Hugh Nagumo Model.R")

### The below function contains a loop that spawns as many datasets as the
	### variable "number" is set at. It takes the arguments xInit (initial
	### species values), t (timepoints), params (parameter values), noise (
	### the standard deviation of the Gaussian noise to add to the data,
	### in order to perturb the data. The save location should be
	### changed depending on where the file should be saved to and which
	### computer operating system is being used (Windows requires the use of
	### .R at the end of the file name, in order to allow the file to save
	### in the correct format)


createDatasets <- function(xInit,t,params,noise,number,averageSNR)
{
	for(i in 1:number)
	{
		ode.data <- simulateFitzHughNagumoModel(xInit,t,params)
		data.true <- ode.data[,2:dim(ode.data)[2]]
	

		data.full <- data.true
		data <- data.true

	for(species in 1:dim(data.true)[2])
	{
      	data[,species] <- data.true[,species]+rnorm(dim(data.true
					)[1],0,noise)
	}



	dataset <- list(data=data,data.true=data.true,time=t,
				params=params,noise=noise)

	save(dataset,file=paste("FhN SD Noise ",noise,
			" Average SNR ",averageSNR," ",i,".R", sep=""))
	}
}


### Specifications of the initial species values, timepoints, parameter
	### values, number of datasets and noise. Can change as desired, below
	### are the default values

speciesNames <- c("V (Voltage)","R (Recovery Variable)")


speciesInitial <- c(-1,-1)


names(speciesInitial) <- speciesNames



t <- seq(0,10,length.out=25)


paramNames <- c("Alpha","Beta","Gamma")


params <- c(0.2,0.2,3)


names(params) <- paramNames

number <- 10

noise <- 0.38
averageSNR <- 10

#noise <- 0.535
#averageSNR <- 5




### Creating the datasets for the Fitz-Hugh Nagumo model

createDatasets(speciesInitial,t,params,noise,number,averageSNR)






