### This script creates the Vyshemirsky And Girolami protein signalling
	### transduction pathway model 1. It sets up the odes and begins
	### preparation for the MCMC to be set-up/run. NOTE:
	### Correction has been made from original publication and this model
	### follows the CORRECTED description


### Load the "deSolve" package

library(deSolve)


### Creating the system of ordinary differential equations for the V&G
	### model 1

### Species: There are 5 species - 2 species from model 4 given constant
	### gradient

### Species in order are:
###  1. S protein
###  2. DS degraded S protein
###  3. R inactive protein
###  4. RS binded protein (R and S proteins)
###  5. Rpp active state protein


### Parameters: There are 6 kinetic parameters

### Parameters in order are:
### 1. k1
### 2. k2
### 3. k3
### 4. k4
### 5. V
### 6. Km


### Calculating gradients for the VG model. This consists of 
	### setting up the system of odes. dydt contains the values of dy by dt
	### (differentiating y with respect to t, where y is the response i.e.
	### the species, and t is time). Species2 stands for the variable X, 
	### and params is a vector of the parameter values

gradientsVGModel1 <- function(params,Species2,timepoints=NULL,auxVars,species
					  )
{
	MM <- ((params[5]*Species2[,5])/(params[6]+Species2[,5]))

	dydt <- switch(species,

		- params[1]*Species2[,1] - params[2]*Species2[,1]*Species2[,3] + 
		params[3]*Species2[,4]
	,
		params[1]*Species2[,1]
	,
		- params[2]*Species2[,1]*Species2[,3] + params[3]*Species2[,4] + 
		MM
	,
		params[2]*Species2[,1]*Species2[,3] - params[3]*Species2[,4] - 
		params[4]*Species2[,4]
	,
		params[4]*Species2[,4] - MM

			  )

	return(dydt)
}


### Creating function odeVG. This will be called in a later stage,
	### during the population MCMC stage and is similar to the gradient
	### specification above. If using the function "c", then each element
	### will be one equation. Species here stands for the variable X (or x)

odeVGModel1 <- function(t,Species,params)
{

	MM <- ((params[5]*Species[5])/(params[6]+Species[5]))

	dydt <- c(

		- params[1]*Species[1] - params[2]*Species[1]*Species[3] + params[
		3]*Species[4]
	,
		params[1]*Species[1]
	,
		- params[2]*Species[1]*Species[3] + params[3]*Species[4] + MM
	,
		params[2]*Species[1]*Species[3] - params[3]*Species[4] - params[4]
		*Species[4]
	,
		params[4]*Species[4] - MM

			 )

	grads = list()
	grads[[1]] = dydt

  	return(grads)
}


### Creation of simulation VG model 1. It uses the function "ode" in the
	### cran package "deSolve" to solve the odeFitzHughNagumo model function
	### above. xInit is the variable containing the initial values for the
	### species concentrations, t is the timepoints at which each species is
	### simulated and params is the vector of parameter values

simulateVGModel1 <- function(xInit,t,params,...)
{
	solution <- ode(xInit,t,odeVGModel1,params,...)

	return(solution)
}




